<?php
/**
 * Plugin Stats View
 *
 * @package    Plugin Stats View
 * @subpackage PluginStatsView Main Functions
/*
	Copyright (c) 2016- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

$pluginstatsview = new PluginStatsView();

/** ==================================================
 * Main Functions
 */
class PluginStatsView {

	/** ==================================================
	 * Construct
	 *
	 * @since 1.14
	 */
	public function __construct() {

		add_filter( 'block_categories', array( $this, 'plugin_stats_view_category' ), 10, 2 );
		add_action( 'init', array( $this, 'psview_block_init' ) );
		add_action( 'init', array( $this, 'paview_block_init' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'dashicon_show_load_dashicons' ) );

	}

	/** ==================================================
	 * Original category for block
	 *
	 * @param array  $categories  categories.
	 * @param object $post  post.
	 * @since 2.10
	 */
	public function plugin_stats_view_category( $categories, $post ) {
		return array_merge(
			$categories,
			array(
				array(
					'slug' => 'plugin-stats-view',
					'title' => 'Plugin Stats View',
					'icon' => 'chart-line',
				),
			)
		);
	}

	/** ==================================================
	 * Attribute block
	 *
	 * @since 2.00
	 */
	public function psview_block_init() {

		$asset_file = include( plugin_dir_path( __DIR__ ) . 'block/dist/psview/psview-block.asset.php' );

		wp_register_script(
			'psview-block',
			plugins_url( 'block/dist/psview/psview-block.js', dirname( __FILE__ ) ),
			$asset_file['dependencies'],
			$asset_file['version'],
			true
		);

		wp_localize_script(
			'psview-block',
			'psview_text',
			array(
				'single'   => __( 'Single Plugin', 'plugin-stats-view' ),
				'slug'     => __( 'Slug' ),
				'settings' => __( 'Other settings', 'plugin-stats-view' ),
				'view'     => __( 'View' ),
				'normal'   => __( 'Normal display', 'plugin-stats-view' ),
				'card'     => __( 'Card display', 'plugin-stats-view' ),
				'simple'   => __( 'Simple display', 'plugin-stats-view' ),
				'link'     => __( 'Link' ),
				'open'     => __( 'View Description', 'plugin-stats-view' ),
			)
		);

		register_block_type(
			'plugin-stats-view/psview-block',
			array(
				'editor_script'   => 'psview-block',
				'render_callback' => array( $this, 'psview_shortcode' ),
				'attributes'      => array(
					'slug'  => array(
						'type'      => 'string',
						'default'   => null,
					),
					'view' => array(
						'type'      => 'string',
						'default'   => 'normal',
					),
					'link' => array(
						'type'      => 'string',
						'default'   => null,
					),
					'open' => array(
						'type'      => 'boolean',
						'default'   => false,
					),
				),
			)
		);

		add_shortcode( 'psview', array( $this, 'psview_shortcode' ) );

	}

	/** ==================================================
	 * Attribute block
	 *
	 * @since 2.10
	 */
	public function paview_block_init() {

		$asset_file = include( plugin_dir_path( __DIR__ ) . 'block/dist/paview/paview-block.asset.php' );

		wp_register_script(
			'paview-block',
			plugins_url( 'block/dist/paview/paview-block.js', dirname( __FILE__ ) ),
			$asset_file['dependencies'],
			$asset_file['version'],
			true
		);

		wp_localize_script(
			'paview-block',
			'paview_text',
			array(
				'multi'    => __( 'Multi Plugins', 'plugin-stats-view' ),
				'slug'     => __( 'Slug' ),
				'settings' => __( 'Other settings', 'plugin-stats-view' ),
				'link'     => __( 'Link' ),
			)
		);

		register_block_type(
			'plugin-stats-view/paview-block',
			array(
				'editor_script'   => 'paview-block',
				'render_callback' => array( $this, 'paview_shortcode' ),
				'attributes'      => array(
					'slug'  => array(
						'type'      => 'string',
						'default'   => null,
					),
					'link' => array(
						'type'      => 'string',
						'default'   => null,
					),
				),
			)
		);

		add_shortcode( 'paview', array( $this, 'paview_shortcode' ) );

	}

	/** ==================================================
	 * Short code
	 *
	 * @param array  $atts  atts.
	 * @param string $content  content.
	 * @return string $content
	 * @since 1.00
	 */
	public function psview_shortcode( $atts, $content = null ) {

		$a = shortcode_atts(
			array(
				'slug'  => '',
				'view' => '',
				'link' => '',
				'open' => '',
			),
			$atts
		);

		return $this->single_stats( $a, $content = null );

	}

	/** ==================================================
	 * Short code
	 *
	 * @param array  $atts  atts.
	 * @param string $content  content.
	 * @return string $content
	 * @since 1.00
	 */
	public function paview_shortcode( $atts, $content = null ) {

		$a = shortcode_atts(
			array(
				'slug'  => '',
				'link' => '',
			),
			$atts
		);

		wp_enqueue_style( 'paview', plugin_dir_url( __DIR__ ) . 'css/paview.css', array(), '1.00' );

		return $this->all_stats( $a, $content = null );

	}

	/** ==================================================
	 * Single stats
	 *
	 * @param array  $a  a.
	 * @param string $content  content.
	 * @return string $content
	 * @since 1.00
	 */
	public function single_stats( $a, $content = null ) {

		$slug = $a['slug'];
		$view = $a['view'];
		$link = $a['link'];
		$open = $a['open'];

		if ( ! empty( $slug ) ) {

			$call_apis = $this->cache( $slug );

			if ( ! empty( $call_apis ) ) {
				if ( is_null( $view ) ) {
					$view = 'normal';
				}
				if ( ! empty( $link ) ) {
					if ( false !== strpos( $link, '%slug%' ) ) {
						$homepage = str_replace( '%slug%', $slug, $link );
					} else {
						$homepage = $link;
					}
				} else {
					$homepage = $call_apis['homepage'];
				}

				$author_link = str_replace( '<a href', '<a style="text-decoration: none;" href', $call_apis['author'] );
				$lastupdated = human_time_diff( strtotime( $call_apis['last_updated'] ), time() ) . __( ' ago', 'plugin-stats-view' );
				/* translators: stars */
				$ratings_text = sprintf( __( '%1$s out of %2$d stars', 'plugin-stats-view' ), number_format( $call_apis['rating'], 1 ), 5 );
				$num_ratings = null;
				if ( 'simple' <> $view ) {
					$num_ratings = '(' . $call_apis['num_ratings'] . ')';
				}
				$rating = '<a title="' . $ratings_text . '">' . $this->ratings( $call_apis['rating'] ) . '</a>' . $num_ratings;

				$active_installs = null;
				if ( $call_apis['active_installs'] > 10 && $call_apis['active_installs'] < 1000000 ) {
					$active_installs .= number_format( $call_apis['active_installs'] ) . __( '+ ', 'plugin-stats-view' ) . __( 'active installs', 'plugin-stats-view' );
				} else if ( $call_apis['active_installs'] >= 1000000 ) {
					/* translators: active install */
					$active_installs .= sprintf( __( '%1$d+ million', 'plugin-stats-view' ), floor( $call_apis['active_installs'] / 1000000 ) ) . __( 'active installs', 'plugin-stats-view' );
				} else {
					$active_installs .= number_format( $call_apis['downloaded'] ) . __( 'Download', 'plugin-stats-view' );
				}
				$active_installs .= "\n";

				if ( 'simple' === $view ) {
					$content .= '<div style="padding:10px; font-size: small;">' . "\n";
				} else if ( 'card' === $view ) {
					$content .= '<div style="padding:10px;">' . "\n";
				} else {
					$content .= '<div>' . "\n";
				}

				if ( 'normal' === $view ) {
					$content .= '<div><img src="' . array_pop( $call_apis['banners'] ) . '" />' . "\n";
				}
				if ( 'card' === $view ) {
					$content .= '<div><img src="' . array_pop( $call_apis['icons'] ) . '" style="width: 128px; height: 128px; float: left; padding: 5px;" />' . "\n";
				} else {
					$content .= '<div><img src="' . array_pop( $call_apis['icons'] ) . '" style="width: 80px; height: 80px; float: left; padding: 5px;" />' . "\n";
				}
				$content .= '<div style="overflow: hidden;">' . "\n";
				$content .= '<div style="font-weight: bold;"><a href="' . $homepage . '" style="text-decoration: none;">' . $call_apis['name'] . '</a></div>' . "\n";
				if ( 'normal' === $view ) {
					$content .= '<div>' . $rating . '</div>' . "\n";
					$content .= '<div>' . __( 'Version', 'plugin-stats-view' ) . $call_apis['version'] . '</div>' . "\n";
				} else if ( 'card' === $view ) {
					$description = $call_apis['short_description'];
					$plus_str = null;
					if ( function_exists( 'mb_substr' ) ) {
						if ( 64 < mb_strlen( $description ) ) {
							$plus_str = '...';
						}
						$description = mb_substr( wp_strip_all_tags( $description, true ), 0, 64 ) . $plus_str;
					} else {
						if ( 64 < strlen( $description ) ) {
							$plus_str = '...';
						}
						$description = substr( wp_strip_all_tags( $description, true ), 0, 64 ) . $plus_str;
					}
					$content .= '<div style="font-size: small;">' . $description . '</div>' . "\n";
					$content .= '<div style="font-size: small;">' . __( 'Author', 'plugin-stats-view' ) . ': ' . $author_link . '</div>' . "\n";
					$content .= '<div style="clear: both;"></div>' . "\n";
				} else if ( 'simple' === $view ) {
					$content .= '<div style="font-size: small;">' . $rating . '</div>' . "\n";
					$content .= '<div style="font-size: small;">' . $active_installs . '</div>' . "\n";
				}
				$content .= '</div></div>' . "\n";
				$content .= '<div style="clear: both;"></div>' . "\n";

				if ( 'card' === $view ) {
					$content .= '<div style="font-size: small;">' . "\n";
					$content .= '<span style="display:inline-block; width:50%;">' . $rating . '</span>' . "\n";
					$content .= '<span style="display:inline-block; width:50%; float: right; text-align:right;">' . __( 'Last Updated', 'plugin-stats-view' ) . ': ' . $lastupdated . '</span>' . "\n";
					$content .= '</div>' . "\n";
					$content .= '<div style="font-size: small;">' . "\n";
					$content .= '<span style="display: inline-block; width:50%;">' . $active_installs . '</span>' . "\n";
					$content .= '<span style="display: inline-block; width:50%; float: right; text-align:right;">' . __( 'Tested up to', 'plugin-stats-view' ) . ': ' . $call_apis['tested'] . '</span>' . "\n";
					$content .= '</div>' . "\n";
					$content .= '<div style="font-size: small;">' . "\n";
					$content .= '<span style="display: inline-block; width:50%;">' . __( 'Download', 'plugin-stats-view' ) . ': <a href="' . $call_apis['download_link'] . '" class="dashicons dashicons-download" style="text-decoration: none; display:inline-block; vertical-align:middle;"></a></span>' . "\n";
					$content .= '<span style="display: inline-block; width:50%; float: right; text-align:right;">' . __( 'Release', 'plugin-stats-view' ) . ': ' . $call_apis['added'] . '</span>' . "\n";
					$content .= '</div>' . "\n";
				}

				if ( 'normal' === $view ) {
					$content .= '<div>' . $active_installs . '</div>' . "\n";
					$content .= '<div>' . $call_apis['short_description'] . '</div>' . "\n";
					$content .= '<details open style="margin-bottom: 5px;">' . "\n";
					$content .= '<summary style="font-weight: bold; cursor: pointer; padding: 10px; border: 1px solid #ddd; background: #f4f4f4; color: #000;">' . __( 'Specification', 'plugin-stats-view' ) . '</summary>' . "\n";
					if ( wp_is_mobile() ) {
						$content .= '<details style="margin-bottom: 5px; text-indent: 1em;">' . "\n";
						$content .= '<summary style="font-weight: bold; cursor: pointer; padding: 10px; border: 1px solid #ddd; background: #f4f4f4; color: #000;">' . __( 'Date' ) . '</summary>' . "\n";
						$content .= '<span style="text-indent: 2em;">';
						$content .= '<div>' . __( 'Author', 'plugin-stats-view' ) . ': ' . $author_link . '</div>' . "\n";
						$content .= '<div>' . __( 'Last Updated', 'plugin-stats-view' ) . ': ' . $lastupdated . '</div>' . "\n";
						$content .= '<div>' . __( 'Release', 'plugin-stats-view' ) . ': ' . $call_apis['added'] . '</div>' . "\n";
						$content .= '</span>';
						$content .= '</details>' . "\n";
						$content .= '<details style="margin-bottom: 5px; text-indent: 1em;">' . "\n";
						$content .= '<summary style="font-weight: bold; cursor: pointer; padding: 10px; border: 1px solid #ddd; background: #f4f4f4; color: #000;">' . __( 'Compatible version', 'plugin-stats-view' ) . '</summary>' . "\n";
						$content .= '<span style="text-indent: 2em;">';
						$content .= '<div><span style="font-weight: bold;">' . __( 'PHP', 'plugin-stats-view' ) . ': </span>' . $call_apis['requires_php'] . '</div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'WordPress', 'plugin-stats-view' ) . ': </span>' . $call_apis['requires'] . '</div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Tested up to', 'plugin-stats-view' ) . ': </span>' . $call_apis['tested'] . '</div>' . "\n";
						$content .= '</span>';
						$content .= '</details>' . "\n";
						$content .= '<details style="margin-bottom: 5px; text-indent: 1em;">' . "\n";
						$content .= '<summary style="font-weight: bold; cursor: pointer; padding: 10px; border: 1px solid #ddd; background: #f4f4f4; color: #000;">' . __( 'Links' ) . '</summary>' . "\n";
						$content .= '<span style="text-indent: 2em;">';
						$content .= '<div><span style="font-weight: bold;">WordPress : </span><a href="' . $call_apis['homepage'] . '" class="dashicons dashicons-wordpress" style="text-decoration: none; display:inline-block; vertical-align:middle;"></a></div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Download', 'plugin-stats-view' ) . ': </span><a href="' . $call_apis['download_link'] . '" class="dashicons dashicons-download" style="text-decoration: none; display:inline-block; vertical-align:middle;"></a></div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Donation', 'plugin-stats-view' ) . ': </span><a href="' . $call_apis['donate_link'] . '" class="dashicons dashicons-external" style="text-decoration: none; display:inline-block; vertical-align:middle;"></a></div>' . "\n";
						$content .= '</span>';
						$content .= '</details>' . "\n";
					} else {
						$content .= '<table style="border-collapse: collapse; width: 100%; table-layout: fixed;">' . "\n";
						$content .= '<tr style="text-align: left;">' . "\n";
						$content .= '<th style="border: 1px solid #b9b9b9; width: 33%;">' . __( 'Date' ) . '</th>' . "\n";
						$content .= '<th style="border: 1px solid #b9b9b9; width: 33%;">' . __( 'Compatible version', 'plugin-stats-view' ) . '</th>' . "\n";
						$content .= '<th style="border: 1px solid #b9b9b9; width: 33%;">' . __( 'Links' ) . '</th>' . "\n";
						$content .= '</tr>' . "\n";
						$content .= '<tr style="text-align: left; vertical-align: top;">' . "\n";
						$content .= '<td style="border: 1px solid #b9b9b9;">' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Author', 'plugin-stats-view' ) . ': </span>' . $author_link . '</div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Last Updated', 'plugin-stats-view' ) . ': </span>' . $lastupdated . '</div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Release', 'plugin-stats-view' ) . ': </span>' . $call_apis['added'] . '</div>' . "\n";
						$content .= '</td>' . "\n";
						$content .= '<td style="border: 1px solid #b9b9b9;">' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'PHP', 'plugin-stats-view' ) . ': </span>' . $call_apis['requires_php'] . '</div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'WordPress', 'plugin-stats-view' ) . ': </span>' . $call_apis['requires'] . '</div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Tested up to', 'plugin-stats-view' ) . ': </span>' . $call_apis['tested'] . '</div>' . "\n";
						$content .= '</td>' . "\n";
						$content .= '<td style="border: 1px solid #b9b9b9;">' . "\n";
						$content .= '<div><span style="font-weight: bold;">WordPress : </span><a href="' . $call_apis['homepage'] . '" class="dashicons dashicons-wordpress" style="text-decoration: none; display:inline-block; vertical-align:middle;"></a></div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Download', 'plugin-stats-view' ) . ': </span><a href="' . $call_apis['download_link'] . '" class="dashicons dashicons-download" style="text-decoration: none; display:inline-block; vertical-align:middle;"></a></div>' . "\n";
						$content .= '<div><span style="font-weight: bold;">' . __( 'Donation', 'plugin-stats-view' ) . ': </span><a href="' . $call_apis['donate_link'] . '" class="dashicons dashicons-external" style="text-decoration: none; display:inline-block; vertical-align:middle;"></a></div>' . "\n";
						$content .= '</td>' . "\n";
						$content .= '</tr>' . "\n";
						$content .= '</table>' . "\n";
					}
					$content .= '</details>' . "\n";
					if ( 'true' === $open || $open ) {
						$content .= '<details open style="margin-bottom: 5px;">' . "\n";
					} else {
						$content .= '<details style="margin-bottom: 5px;">' . "\n";
					}
					$content .= '<summary style="font-weight: bold; cursor: pointer; padding: 10px; border: 1px solid #ddd; background: #f4f4f4; color: #000;">' . __( 'Description' ) . '</summary>' . "\n";
					$content .= '<div style="padding: 20px;">' . $call_apis['description'] . '</div>' . "\n";
					$content .= '</details>' . "\n";
				}
				$content .= '</div>' . "\n";

				return $content;
			}
		} else {
			if ( is_user_logged_in() ) {
				$content .= '<div style="text-align: center;">';
				$content .= '<div><strong><span class="dashicons dashicons-chart-line" style="position: relative; top: 5px;"></span>Plugin Stats View</strong></div>';
				/* translators: Input Slug */
				$content .= sprintf( __( 'Please input "%1$s".', 'plugin-stats-view' ), __( 'Slug' ) );
				$content .= '</div>';
				return $content;
			}
		}

	}

	/** ==================================================
	 * All stats
	 *
	 * @param array  $a  a.
	 * @param string $content  content.
	 * @return string $content
	 * @since 1.00
	 */
	public function all_stats( $a, $content = null ) {

		$link = $a['link'];

		if ( ! empty( $a['slug'] ) ) {
			$slugs = explode( ',', $a['slug'] );
			sort( $slugs );
			$active_installs = null;
			$downloadeds = null;
			$all_ratings = array(
				5 => 0,
				4 => 0,
				3 => 0,
				2 => 0,
				1 => 0,
			);
			$full_rate_count = 0;
			$count = 0;
			foreach ( $slugs as $slug ) {
				$call_apis = $this->cache( $slug );
				if ( ! empty( $call_apis ) ) {
					$content .= '<div style="padding:10px; font-size: small; width: 320px; height: 80px; float: left;">' . "\n";
					$content .= '<div><img src="' . array_pop( $call_apis['icons'] ) . '" style="width: 80px; height: 80px; float: left; padding: 5px;" />' . "\n";
					$content .= '<div style="overflow: hidden;">' . "\n";
					if ( ! empty( $link ) ) {
						if ( false !== strpos( $link, '%slug%' ) ) {
							$homepage = str_replace( '%slug%', $slug, $link );
						} else {
							$homepage = $link;
						}
					} else {
						$homepage = $call_apis['homepage'];
					}
					$content .= '<div style="font-weight: bold;"><a href="' . $homepage . '" style="text-decoration: none;">' . $call_apis['name'] . '</a></div>' . "\n";
					/* translators: stars */
					$ratings_text = sprintf( __( '%1$s out of %2$d stars', 'plugin-stats-view' ), number_format( $call_apis['rating'], 1 ), 5 );
					$content .= '<div><a title="' . $ratings_text . '">' . $this->ratings( $call_apis['rating'] ) . '</a></div>' . "\n";
					$content .= '<div>';
					if ( $call_apis['active_installs'] > 10 && $call_apis['active_installs'] < 1000000 ) {
						$content .= number_format( $call_apis['active_installs'] ) . __( '+ ', 'plugin-stats-view' ) . __( 'active installs', 'plugin-stats-view' );
					} else if ( $call_apis['active_installs'] >= 1000000 ) {
						/* translators: active install */
						$content .= sprintf( __( '%1$d+ million', 'plugin-stats-view' ), floor( $call_apis['active_installs'] / 1000000 ) ) . __( 'active installs', 'plugin-stats-view' );
					} else {
						$content .= number_format( $call_apis['downloaded'] ) . __( 'Download', 'plugin-stats-view' );
					}
					$content .= '</div>' . "\n";
					$content .= '</div></div>' . "\n";
					$content .= '</div>' . "\n";

					$active_installs += $call_apis['active_installs'];
					$downloadeds += $call_apis['downloaded'];
					foreach ( $call_apis['ratings'] as $rate_no => $rate_count ) {
						$all_ratings[ $rate_no ] += $rate_count;
						$full_rate_count += $rate_count;
					}
					++$count;
				}
			}
			$content .= '<div style="clear: both;"></div>' . "\n";

			$content .= '<div style="text-align: center; font-weight: bold; font-size: large;">';
			/* translators: Plugins count */
			$content .= '<div>' . sprintf( __( '%1$d Plugins', 'plugin-stats-view' ), number_format( $count ) ) . '</div>';
			$content .= '<div>' . number_format( $active_installs ) . __( '+ ', 'plugin-stats-view' ) . __( 'active installs', 'plugin-stats-view' ) . '</div>';
			$content .= '<div>' . number_format( $downloadeds ) . __( 'Download', 'plugin-stats-view' ) . '</div>';
			$content .= '</div>' . "\n";
			$content .= '<div align="center">' . "\n";
			$content .= '<div style="text-align: center; font-weight: bold; font-size: large;">';
			$content .= __( 'Reviews count', 'plugin-stats-view' ) . "\n";
			$content .= '</div>' . "\n";
			if ( 0 < $full_rate_count ) {
				foreach ( $all_ratings as $rate_no => $all_rate_count ) {
					$stars = __( 'stars', 'plugin-stats-view' );
					if ( 1 == $rate_no ) {
						$stars = __( 'star ', 'plugin-stats-view' );
					}
					$content .= '<div style="line-height: 1em;">' . "\n";
					$content .= '<span style="width: 4em; display: inline-block; text-align: left;">' . "\n";
					$content .= $rate_no . $stars;
					$content .= '</span>' . "\n";
					$content .= '<span style="width: 6em; display: inline-block;">' . "\n";
					$all_per = round( $all_rate_count / $full_rate_count * 100 );
					$content .= '<progress class="paview" value="' . $all_per . '" max="100"></progress>';
					$content .= '</span>' . "\n";
					$content .= '<span style="width: 4em; display: inline-block; text-align: left;">' . "\n";
					$content .= ' ' . number_format( $all_rate_count );
					$content .= '</span>' . "\n";
					$content .= '</div>' . "\n";
				}
				$content .= '<div style="text-align: center; font-weight: bold; font-size: large;">';
				$content .= __( 'Reviews total', 'plugin-stats-view' ) . "\n";
				$content .= number_format( $full_rate_count );
				$content .= '</div>' . "\n";
			}
			$content .= '</div>' . "\n";
			return $content;
		} else {
			if ( is_user_logged_in() ) {
				$content .= '<div style="text-align: center;">';
				$content .= '<div><strong><span class="dashicons dashicons-chart-line" style="position: relative; top: 5px;"></span>Plugin Stats View</strong></div>';
				/* translators: Input Slugs */
				$content .= sprintf( __( 'Please input "%1$s" separated by commas.', 'plugin-stats-view' ), __( 'Slug' ) );
				$content .= '</div>';
				return $content;
			}
		}

	}

	/** ==================================================
	 * Cache read or create
	 *
	 * @param string $slug  slug.
	 * @return array $call_apis
	 * @since 1.00
	 */
	private function cache( $slug ) {

		$call_apis = array();
		if ( get_transient( 'psview_datas_' . $slug . '_' . get_locale() ) ) {
			/* Get cache */
			$call_apis = get_transient( 'psview_datas_' . $slug . '_' . get_locale() );
		} else {
			/* Call API */
			require_once ABSPATH . 'wp-admin/includes/plugin-install.php';
			$call_api = plugins_api(
				'plugin_information',
				array(
					'slug' => $slug,
					'fields' => array(
						'short_description' => true,
						'description' => true,
						'active_installs' => true,
						'downloaded' => true,
						'icons' => true,
						'banners' => true,
						'ratings' => true,
					),
				)
			);
			if ( is_wp_error( $call_api ) ) {
				$dummy = 0; /* skip */
			} else {
				$call_apis = array(
					'name' => $call_api->name,
					'icons' => $call_api->icons,
					'banners' => $call_api->banners,
					'short_description' => $call_api->short_description,
					'description' => $call_api->description,
					'author' => $call_api->author,
					'version' => $call_api->version,
					'homepage' => $call_api->homepage,
					'name' => $call_api->name,
					'added' => $call_api->added,
					'requires' => $call_api->requires,
					'requires_php' => $call_api->requires_php,
					'tested' => $call_api->tested,
					'last_updated' => $call_api->last_updated,
					'rating' => 5 * ( $call_api->rating / 100 ),
					'num_ratings' => $call_api->num_ratings,
					'ratings' => $call_api->ratings,
					'downloaded' => $call_api->downloaded,
					'download_link' => $call_api->download_link,
					'donate_link' => $call_api->donate_link,
					'active_installs' => $call_api->active_installs,
				);

				/* Set cache */
				set_transient( 'psview_datas_' . $slug . '_' . get_locale(), $call_apis, 86400 );
			}
		}

		return $call_apis;

	}

	/** ==================================================
	 * Rating 5 stars
	 *
	 * @param float $score  score.
	 * @return string  $stars  stars.
	 * @since 1.00
	 */
	private function ratings( $score ) {

		if ( 0 <= $score && 0.25 > $score ) {
			$stars = '<span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 0.25 <= $score && 0.75 > $score ) {
			$stars = '<span class="dashicons dashicons-star-half"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 0.75 <= $score && 1.25 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 1.25 <= $score && 1.75 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-half"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 1.75 <= $score && 2.25 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 2.25 <= $score && 2.75 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-half"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 2.75 <= $score && 3.25 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-empty"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 3.25 <= $score && 3.75 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-half"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 3.75 <= $score && 4.25 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-empty"></span>';
		} else if ( 4.25 <= $score && 4.75 > $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-half"></span>';
		} else if ( 4.75 <= $score ) {
			$stars = '<span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span><span class="dashicons dashicons-star-filled"></span>';
		}

		return '<span style="color: #ffcc33;">' . $stars . '</span>';

	}

	/** ==================================================
	 * Dashicon
	 *
	 * @since 2.00
	 */
	public function dashicon_show_load_dashicons() {
		wp_enqueue_style( 'dashicons' );
	}

}


